from mixcoatl.resource import Resource
from mixcoatl.decorators.validations import required_attrs
from mixcoatl.decorators.lazy import lazy_property
from mixcoatl.utils import camelize, camel_keys, uncamel_keys
import json


class MachineImage(Resource):

    """ A machine image is the baseline image or template from which virtual machines may be
    provisioned. Some clouds allow machine image/template sharing. In those clouds, Dell Cloud
    Manager creates multiple machine image records referencing the shared machine image object
    to enable users to maintain separate meta-data over those shared images. """
    PATH = 'infrastructure/MachineImage'
    COLLECTION_NAME = 'images'
    PRIMARY_KEY = 'machine_image_id'

    def __init__(self, machine_image_id=None, endpoint=None, *args, **kwargs):
        """A machine image is the baseline image or template from which
            virtual machines may be provisioned.
        """
        Resource.__init__(self, request_details='basic', endpoint=endpoint)
        self.__machine_image_id = machine_image_id

    @property
    def machine_image_id(self):
        """`int` - The unique DCM id for this machine image"""
        return self.__machine_image_id

    @lazy_property
    def architecture(self):
        """`str` - The underlying CPU architecture of the virtual machine in question"""
        return self.__architecture

    @lazy_property
    def legacy_owner_id(self):
        return self.__legacy_owner_id

    @lazy_property
    def cloud(self):
        """`dict` - The cloud in which this machine image is installed"""
        return self.__cloud

    @lazy_property
    def creation_timestamp(self):
        """`str` - The date and time this machine image was first created

        .. .note::

                Some clouds do not report his value and it may therefore be `00:00 UTC January 1, 1970

        """
        return self.__creation_timestamp

    @lazy_property
    def customer(self):
        """`dict` - The customer in whose library this machine image record is being managed"""
        return self.__customer

    @lazy_property
    def budget(self):
        """`int` - The id of the billing code against which costs associated are
            billed for this machine image
        """
        return self.__budget

    @lazy_property
    def name(self):
        """`str` - The user friendly name for the machine image"""
        return self.__name

    @name.setter
    def name(self, n):
        self.__name = n

    @lazy_property
    def description(self):
        """`str` - The description of the machine image established in DCM"""
        return self.__description

    @description.setter
    def description(self, d):
        self.__description = d

    @lazy_property
    def owning_account(self):
        """`dict` - The DCM cloud account that is the account under which
            the machine image is registered

            .. .note::

                This value may be empty if the machine image belongs to an account
                not using DCM
        """
        return self.__owning_account

    @lazy_property
    def owning_cloud_account_number(self):
        """`str` - The DCM cloud account that is the account under which
            the machine image is registered.

            .. .note::

                This value is empty for public images and in rare circumstances
                when DCM is unable to determine the ownership

        """
        return self.__owning_cloud_account_number

    @lazy_property
    def owning_user(self):
        """`dict` or `None` - The user who is the owner of record of this machine image.

            .. .note::

                The owner may be null in cases of auto-discovery or certain automated scenarios

        """
        return self.__owning_user

    @lazy_property
    def owning_groups(self):
        """`list` - The groups who have ownership over this machine image"""
        return self.__owning_groups

    @owning_groups.setter
    def owning_groups(self, d):
        self.__owning_groups = d

    @lazy_property
    def platform(self):
        """`str` - The operating system bundled into the machine image/template"""
        return self.__platform

    @lazy_property
    def provider_id(self):
        """`str` - The cloud provider's unique id for the machine image"""
        return self.__provider_id

    @lazy_property
    def region(self):
        """`dict` - The region in which this machine image is available"""
        return self.__region

    @lazy_property
    def removable(self):
        """`bool` - Whether or not this machine image can be deleted"""
        return self.__removable

    @lazy_property
    def sharable(self):
        """`bool` - Whether or not this image can be shared"""
        return self.__sharable

    @lazy_property
    def status(self):
        """`str` - The current status of this machine image"""
        return self.__status

    @lazy_property
    def label(self):
        """`str` - A color label assigned to this machine image"""
        return self.__label

    @lazy_property
    def products(self):
        """`list` - The server products that can be used to provision a virtual
            machine based on this machine image/template
        """
        return self.__products

    @lazy_property
    def agent_version(self):
        """`int` - The version of the DCM agent if installed on the machine image"""
        return self.__agent_version

    @lazy_property
    def public(self):
        """`bool` - Indicates whether or not this image is publicly shared. 
        This value may be modified only for machine images that belong to your account. """
        return self.__public

    @public.setter
    def public(self, p):
        self.__public = p

    @lazy_property
    def budget(self):
        return self.__budget

    @budget.setter
    def budget(self, b):
        self.__budget = b

    @required_attrs(['machine_image_id'])
    def destroy(self, reason='no reason provided'):
        """Deletes machine image with reason :attr:`reason`

        :param reason: The reason for removing the image
        :type reason: str.
        :returns: bool -- Result of API call
        """
        p = self.PATH + "/" + str(self.machine_image_id)
        qopts = {'reason': reason}
        return self.delete(p, params=qopts)

    @required_attrs(['server_id', 'name', 'budget', 'description', 'owning_groups'])
    def create(self, callback=None):
        """Creates a machine image from server_id

        >>> def cb(j): print(j)
        >>> m = MachineImage()
        >>> m.server_id = 12345
        >>> m.name = 'image-1-test'
        >>> m.budget = 12345
        >>> m.create(callback=cb)

        :returns: int -- The job id of the create request
        """
        payload = {'imageServer':
                   [{
                       'budget': int(self.budget),
                       'description': self.description,
                       'name': self.name,
                       'server': {"serverId": int(self.server_id)},
                       'owningGroups': [{"groupId": self.owning_groups}],
                   }]}

        self.post(data=json.dumps(payload))
        if self.last_error is None:
            if callback is not None:
                callback(self.current_job)
            else:
                return self.current_job
        else:
            raise MachineImageException(self.last_error)

    @required_attrs(['machine_image_id'])
    def update(self, **kwargs):
        """Updates meta-data for an image.

        :param description: The description of an image.
        :type description: str.
        :param name: The name of an image.
        :type name: str.
        :param label: The label of an image.
        :type label: str.
        :returns: True if successful or an error message if fails.
        """
        payload = {'describeImage': [{}]}

        if 'description' in kwargs:
            payload['describeImage'][0]['description'] = kwargs['description']
        if 'name' in kwargs:
            payload['describeImage'][0]['name'] = kwargs['name']
        if 'label' in kwargs:
            payload['describeImage'][0]['label'] = kwargs['label']

        p = self.PATH + "/" + str(self.machine_image_id)
        return self.put(p, data=json.dumps(payload))

    @required_attrs(['machine_image_id'])
    def register_agent(self, agent_version, **kwargs):
        """Registers an agent with the machine image.

        :param agent_version: The agent protocol version to be registered.
        :type name: int.
        :returns: True if successful or an error message if fails.
        """
        payload = {'registerAgent': [{"agentVersion": agent_version}]}
        p = self.PATH + "/" + str(self.machine_image_id)
        return self.put(p, data=json.dumps(payload))

    @required_attrs(['machine_image_id'])
    def unregister_agent(self, **kwargs):
        """Unregisters an agent from the machine image.

        :returns: True if successful or an error message if fails.
        """
        payload = {'unRegisterAgent': [{}]}
        p = self.PATH + "/" + str(self.machine_image_id)
        return self.put(p, data=json.dumps(payload))

    @classmethod
    def all(cls, endpoint=None, **kwargs):
        """Return all machine images

        :param machine_image_id: The id of the machine image
        :type machine_image_id: int.
        :param region_id: The region to search for machine images
        :type region_id: int.
        :param keys_only: Return :attr:`machine_image_id` instead of :class:`MachineImage`
        :type keys_only: bool.
        :param available: Return only available images. Default is `true`
        :type available: str.
        :param registered: Return only images with the DCM agent installed. Default is `false`
        :type registered: str.
        :param detail: The level of detail to return - `basic` or `extended`
        :type detail: str.
        :returns: `list` of :class:`MachineImage` or :attr:`machine_image_id`
        :raises: :class:`MachineImageException`
        """
        if 'machine_image_id' in kwargs:
            r = Resource(cls.PATH + "/" + str(kwargs['machine_image_id']), endpoint=endpoint)
        else:
            r = Resource(cls.PATH, endpoint=endpoint)

        params = {}

        if 'keys_only' in kwargs:
            keys_only = kwargs['keys_only']
        else:
            keys_only = False

        if 'region_id' in kwargs:
            params['regionId'] = kwargs['region_id']

        if 'available' in kwargs:
            params['active'] = kwargs['available']

        if 'registered' in kwargs:
            params['registered'] = kwargs['registered']

        x = r.get(params=params)
        if r.last_error is None:
            if keys_only is True:
                return [i[camelize(cls.PRIMARY_KEY)]
                        for i in x[cls.COLLECTION_NAME]]
            else:
                return [type(cls.__name__, (object,), i)
                        for i in uncamel_keys(x)[cls.COLLECTION_NAME]]
        else:
            raise MachineImageException(r.last_error)


class MachineImageException(BaseException):
    pass



from mixcoatl.resource import Resource
from mixcoatl.decorators.lazy import lazy_property
from mixcoatl.utils import uncamel, camelize, camel_keys, uncamel_keys


class ConfigurationManagementAccount(Resource):

    """ A configuration management account is an account with a configuration management service.
    A typical Dell Cloud Manager installation will have any number of configuration management
    services for different configuration management systems (Chef, Puppet, etc.) installed."""
    PATH = 'automation/ConfigurationManagementAccount'
    COLLECTION_NAME = 'cmAccounts'
    PRIMARY_KEY = 'cm_account_id'

    def __init__(self, cm_account_id=None, endpoint=None, *args, **kwargs):
        Resource.__init__(self, endpoint=endpoint)
        self.__cm_account_id = cm_account_id

    @property
    def cm_account_id(self):
        return self.__cm_account_id

    @lazy_property
    def account_number(self):
        return self.__account_number

    @lazy_property
    def budget(self):
        return self.__budget

    @lazy_property
    def cm_service(self):
        return self.__cm_service

    @lazy_property
    def created_timestamp(self):
        return self.__created_timestamp

    @lazy_property
    def description(self):
        return self.__description

    @lazy_property
    def guid(self):
        return self.__guid

    @lazy_property
    def last_modified_timestamp(self):
        return self.__last_modified_timestamp

    @lazy_property
    def name(self):
        return self.__name

    @lazy_property
    def removable(self):
        return self.__removable

    @lazy_property
    def status(self):
        return self.__status

    @lazy_property
    def customer(self):
        return self.__customer

    @lazy_property
    def label(self):
        return self.__label

    @lazy_property
    def owning_groups(self):
        return self.__owning_groups

    @lazy_property
    def owning_user(self):
        return self.__owning_user

    @classmethod
    def all(cls, endpoint=None, **kwargs):
        r = Resource(cls.PATH, endpoint=endpoint)
        if 'details' in kwargs:
            r.request_details = kwargs['details']
        else:
            r.request_details = 'basic'

        if 'keys_only' in kwargs:
            keys_only = kwargs['keys_only']
        else:
            keys_only = False

        x = r.get()
        if r.last_error is None:
            if keys_only is True:
                return [i[camelize(cls.PRIMARY_KEY)] for i in x[cls.COLLECTION_NAME]]
            else:
                return [type(cls.__name__, (object,), i) for i in uncamel_keys(x)[uncamel(cls.COLLECTION_NAME)]]
        else:
            raise ConfigurationManagementAccountException(r.last_error)


class ConfigurationManagementAccountException(BaseException):
    pass



from mixcoatl.resource import Resource
from mixcoatl.admin.job import Job
from mixcoatl.decorators.lazy import lazy_property
from mixcoatl.utils import uncamel, camelize, camel_keys, uncamel_keys


class RelationalDatabaseProduct(Resource):
    """Represents a product with costs from a cloud relational database vendor."""

    PATH = 'platform/RelationalDatabaseProduct'
    COLLECTION_NAME = 'rdbmsProducts'
    PRIMARY_KEY = 'product_id'

    def __init__(self, product_id=None, endpoint=None, *args, **kwargs):
        Resource.__init__(self, endpoint=endpoint)
        self.__product_id = product_id

    @property
    def product_id(self):
        """`int` - The unique ID in DCM for this relational database product."""
        return self.__product_id

    @lazy_property
    def status(self):
        """`str` - The current status of the product."""
        return self.__status

    @lazy_property
    def architecture(self):
        """`str` - The underlying CPU architecture of this database server."""
        return self.__architecture

    @lazy_property
    def cloud(self):
        """`dict` - The cloud for which this product operates."""
        return self.__cloud

    @lazy_property
    def core_count(self):
        """`int` - The number of CPU cores allocated to your database environment."""
        return self.__core_count

    @lazy_property
    def cpu_in_g_hz(self):
        """`int` - The speed of the CPUs allocated to your database environment."""
        return self.__cpu_in_g_hz

    @lazy_property
    def custom_pricing(self):
        """`bool` - Indicates whether or not this pricing reflects the standard retail rates from the cloud provider."""
        return self.__custom_pricing

    @lazy_property
    def description(self):
        """`str` - A long description for this product."""
        return self.__description

    @lazy_property
    def engine(self):
        """ `enum` - The database engine represented by this product."""
        return self.__engine

    @lazy_property
    def hourly_pricing(self):
        """`dict` - The hourly rate the cloud provider charges for having a relational database provisioned."""
        return self.__hourly_pricing

    @lazy_property
    def io_pricing(self):
        """`dict` - The rate charged by the cloud provider for data going in and out of the cloud to the database."""
        return self.__io_pricing

    @lazy_property
    def io_units(self):
        """`int` - The number of I/O units reflecting in the I/O pricing."""
        return self.__io_units

    @lazy_property
    def maximum_storage_in_gb(self):
        """`int` - The amount of storage up to which you may have allocated to be reflected by this product."""
        return self.__maximum_storage_in_gb

    @lazy_property
    def memory_in_gb(self):
        """`int` - The amount of RAM allocated to this virtual database server."""
        return self.__memory_in_gb

    @lazy_property
    def minimum_storage_in_gb(self):
        """`int` - The amount of storage you must have allocated to receive the pricing reflected in this product."""
        return self.__minimum_storage_in_gb

    @lazy_property
    def name(self):
        """`str` - A user-friendly name to describe this product."""
        return self.__name

    @lazy_property
    def region(self):
        """`dict` - A region for which this product is good."""
        return self.__region

    @lazy_property
    def provider_id(self):
        """`str` - How this product is identified to the cloud provider."""
        return self.__provider_id

    @lazy_property
    def storage_pricing(self):
        """`dict` - The rate per storageUnits charged for the storage allocated for this relational database product."""
        return self.__storage_pricing

    @lazy_property
    def storage_units(self):
        """`int` - The number of storage units reflected in the storage pricing."""
        return self.__storage_units

    def reload(self):
        """Reload resource data from API calls"""
        if self.product_id is not None:
            self.load()
        elif self.current_job is None:
            self.load()
        else:
            if Job.wait_for(self.current_job):
                job = Job(self.current_job, endpoint=self.endpoint)
                self.__relational_database_id = job.message
                self.load()
            else:
                return self.last_error

    @classmethod
    def all(cls, region_id, engine, endpoint=None, **kwargs):
        """Get a list of all known relational_databases

        >>> RelationalDatabaseProduct.all(region_id=100, engine='MYSQL51')
        [{'product_id':1,...},{'product_id':2,...}]

        :returns: list -- a list of :class:`RelationalDatabaseProduct`
        :raises: RelationalDatabaseProductException
        """
        r = Resource(cls.PATH, endpoint=endpoint)
        r.request_details = 'basic'
        params = {'regionId': region_id, 'engine': engine}

        if 'detail' in kwargs:
            r.request_details = kwargs['detail']
        else:
            r.request_details = 'basic'

        if 'keys_only' in kwargs:
            keys_only = kwargs['keys_only']
        else:
            keys_only = False

        x = r.get(params=params)
        if r.last_error is None:
            if keys_only is True:
                results = [i[camelize(cls.PRIMARY_KEY)] for i in x[cls.COLLECTION_NAME]]
            else:
                results = [type(cls.__name__, (object,), i) for i in uncamel_keys(x)[uncamel(cls.COLLECTION_NAME)]]
            return results
        else:
            raise RelationalDatabaseProductException(r.last_error)

class RelationalDatabaseProductException(BaseException):
    pass



"""Implements the DCM Firewall API"""
from mixcoatl.resource import Resource
from mixcoatl.decorators.lazy import lazy_property
from mixcoatl.decorators.validations import required_attrs
from mixcoatl.network.firewall_rule import FirewallRule
from mixcoatl.utils import camelize, camel_keys, uncamel_keys
from mixcoatl.admin.job import Job
import json


class Firewall(Resource):

    """ List Firewalls """
    PATH = 'network/Firewall'
    COLLECTION_NAME = 'firewalls'
    PRIMARY_KEY = "firewall_id"

    def __init__(self, firewall_id=None, endpoint=None, **kwargs):
        Resource.__init__(self, endpoint=endpoint)

        if 'detail' in kwargs:
            self.request_details = kwargs['detail']

        self.__firewall_id = firewall_id

    @property
    def firewall_id(self):
        """`str` - The unique DCM ID for this firewall"""
        return self.__firewall_id

    @lazy_property
    def status(self):
        """`str` - The status of this firewall"""
        return self.__status

    @lazy_property
    def label(self):
        """`str` - A color label assigned to this firewall"""
        return self.__label

    @label.setter
    def label(self, l):
        self.__label = l

    @lazy_property
    def owning_account(self):
        """`dict` - The DCM account under which this firewall is registered"""
        return self.__owning_account

    @lazy_property
    def name(self):
        """`str` - The user-friendly name for this firewall"""
        return self.__name

    @name.setter
    def name(self, n):
        self.__name = n

    @lazy_property
    def owning_user(self):
        """`dict` or `None` - The DCM owner of record for this firewall"""
        return self.__owning_user

    @lazy_property
    def owning_groups(self):
        """`list` - The DCM groups that have ownership of this firewall"""
        return self.__owning_groups

    @lazy_property
    def legacy_owner_id(self):
        """`int` - DCM user ID that represents user ID before 07 DEC 2013."""
        return self.__legacy_owner_id

    @lazy_property
    def description(self):
        """`str` - The description of this firewall in DCM"""
        return self.__description

    @description.setter
    def description(self, d):
        self.__description = d

    @lazy_property
    def cloud(self):
        """`dict` - The DCM cloud account in which this firewall lives"""
        return self.__cloud

    @lazy_property
    def provider_id(self):
        """`str` - The cloud provider unique id for this firewall"""
        return self.__provider_id

    @lazy_property
    def region(self):
        """`dict` - The region in which this firewall operates"""
        return self.__region

    @region.setter
    def region(self, r):
        self.__region = {'region_id': r}

    @lazy_property
    def removable(self):
        """`bool` - Indicates if this firewall can be removed"""
        return self.__removable

    @lazy_property
    def budget(self):
        """`int` - The DCM billing code costs are associated with"""
        return self.__budget

    @budget.setter
    def budget(self, b):
        self.__budget = b

    @lazy_property
    def customer(self):
        """`dict` - The DCM customer to which this firewall belongs"""
        return self.__customer

    @property
    def rules(self):
        """`list` - The firewall rules associated with this firewall"""
        if self.__firewall_id is None:
            self.__rules = []
            return []
        try:
            return self.__rules
        except AttributeError:
            r = FirewallRule.all(
                self.__firewall_id, detail=self.request_details)
            if len(r) < 1:
                self.__rules = []
            else:
                self.__rules = r
            return r

    @required_attrs(['budget', 'region', 'name', 'description'])
    def create(self, **kwargs):
        """Create a new firewall

        :param label: Optional label to assign the firewall
        :type label: str.
        :param callback: Optional callback to call with resulting :class:`Firewall`
        :type callback: func.
        :returns: :class:`Firewall`
        :raises: :class:`FirewallException`
        """

        payload = {'add_firewall': [{
            'budget': self.budget,
            'region': self.region,
            'name': self.name,
            'description': self.description
        }]}

        if 'label' in kwargs:
            payload['add_firewall'][0]['label'] = kwargs['label']

        callback = kwargs.get('callback', None)

        self.post(self.PATH, data=json.dumps(camel_keys(payload)))

        if self.last_error is None:
            if callback is None:
                return self
            else:
                if Job.wait_for(self.current_job) is True:
                    j = Job(self.current_job, endpoint=self.endpoint)
                    self.__firewall_id = j.message
                    self.load()
                    return self
                else:
                    raise FirewallException(j.last_error)
        else:
            raise FirewallException(self.last_error)

    def describe(self):
        """Describe a firewall"""

    @classmethod
    def all(cls, endpoint=None, **kwargs):
        """List all firewalls in `region_id`
        :param region_id: Limit results to `region_id`
        :type region_id: int.
        :param account_id: limit results to `account_id`
        :type account_id: int.
        :param detail: Level of detail to return - `basic` or `extended`
        :type detail: str.
        :param keys_only: Return only :attr:`firewall_id` in results
        :type keys_only: bool.
        :returns: `list` of :attr:`firewall_id` or :class:`Firewall`
        :raises: :class:`FirewallException`
        """
        r = Resource(cls.PATH, endpoint=endpoint)
        params = {}

        if 'detail' in kwargs:
            r.request_details = kwargs['detail']
        else:
            r.request_details = 'basic'

        if 'keys_only' in kwargs:
            keys_only = kwargs['keys_only']
        else:
            keys_only = False

        if 'region_id' in kwargs:
            params['regionId'] = kwargs['region_id']

        if 'account_id' in kwargs:
            params['accountId'] = kwargs['account_id']

        x = r.get(params=params)
        if r.last_error is None:
            if keys_only is True:
                return [i[camelize(cls.PRIMARY_KEY)]
                        for i in x[cls.COLLECTION_NAME]]
            else:
                return [type(cls.__name__, (object,), i)
                        for i in uncamel_keys(x)[cls.COLLECTION_NAME]]
        else:
            raise FirewallException(r.last_error)

    def describe_firewall(firewall_id, endpoint=None, **kwargs):
        """Changes the basic meta-data for a firewall

        :param firewall_id: The id of the firewall to modify
        :type firewall_id: int.
        :param description: The description of the firewall
        :type description: string.
        :param label: The label to assign the firewall.
        :type label: string.
        :param name: The name to give the firewall.
        :type name: string.
        :returns: `bool`
        :raises: :class:`FirewallException`
        """

        if kwargs:
            f = Firewall(firewall_id, endpoint=endpoint)
            f.label = kwargs.get('label', None)
            f.description = kwargs.get('name', None)
            f.label = kwargs.get('label', None)
            return f.describe()
        else:
            raise FirewallException("No attributes were specified to change")


class FirewallException(BaseException):

    """Generic Firewall Exception"""
    pass
